@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bximage.cc 14091 2021-01-30 17:37:42Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2021  The Bochs Project
+//  Copyright (C) 2001-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -55,6 +55,8 @@
 
 #define BX_MAX_CYL_BITS 24 // 8 TB
 
+#define SECTOR_SIZE 512
+
 const int bx_max_hd_megs = (int)(((1 << BX_MAX_CYL_BITS) - 1) * 16.0 * 63.0 / 2048.0);
 
 int  bximage_func;
@@ -71,7 +73,7 @@ char bx_filename_1[512];
 char bx_filename_2[522];
 
 const char *EOF_ERR = "ERROR: End of input";
-const char *svnid = "$Id: bximage.cc 14091 2021-01-30 17:37:42Z sshwarts $";
+const char *svnid = "$Id$";
 const char *divider = "========================================================================";
 const char *main_menu_prompt =
 "\n"
@@ -211,6 +213,7 @@ int ask_int(const char *prompt, int min, int max, int the_default, int *out)
   while (1) {
     printf("%s", prompt);
     printf("[%d] ", the_default);
+    fflush(stdout);
     if (!fgets(buffer, sizeof(buffer), stdin))
       return -1;
     clean = clean_string(buffer);
@@ -240,6 +243,7 @@ int ask_menu(const char *prompt, int n_choices, const char *choice[], int the_de
   while(1) {
     printf("%s", prompt);
     printf("[%s] ", choice[the_default]);
+    fflush(stdout);
     if (!fgets(buffer, sizeof(buffer), stdin))
       return -1;
     clean = clean_string(buffer);
@@ -261,6 +265,7 @@ int ask_menu(const char *prompt, int n_choices, const char *choice[], int the_de
       printf("%s", choice[i]);
     }
     printf("\n");
+    fflush(stdout);
   }
 }
 
@@ -272,6 +277,7 @@ int ask_yn(const char *prompt, int the_default, int *out)
   while (1) {
     printf("%s", prompt);
     printf("[%s] ", the_default?"yes":"no");
+    fflush(stdout);
     if (!fgets(buffer, sizeof(buffer), stdin))
       return -1;
     clean = clean_string(buffer);
@@ -294,6 +300,7 @@ int ask_string(const char *prompt, char *the_default, char *out)
   char *clean;
   printf("%s", prompt);
   printf("[%s] ", the_default);
+  fflush(stdout);
   if (!fgets(buffer, sizeof(buffer), stdin))
     return -1;
   clean = clean_string(buffer);
@@ -548,8 +555,8 @@ void print_usage()
     "  -fd=...       create: floppy image with size code\n"
     "  -hd=...       create/resize: hard disk image with size in megabytes (M)\n"
     "                or gigabytes (G)\n"
-    "  -imgmode=...  create/convert: hard disk image mode\n"
-    "  -sectsize=... create: hard disk sector size\n"
+    "  -imgmode=...  create/convert: hard disk image mode (default = flat)\n"
+    "  -sectsize=... create: hard disk sector size (default = 512)\n"
     "  -b            convert/resize: create a backup of the source image\n"
     "                commit: create backups of the base image and redolog file\n"
     "  -q            quiet mode (don't prompt for user input)\n"
@@ -704,11 +711,13 @@ int parse_cmdline(int argc, char *argv[])
     arg++;
   }
   if (bximage_func == BXIMAGE_FUNC_NULL) {
+    printf("\nERROR: Parameter -func missing - switching to interactive mode.\n\n");
     bx_interactive = 1;
   } else {
     set_default_values();
     if (fnargs < 1) {
       bx_interactive = 1;
+      printf("\nERROR; Filename missing - switching to interactive mode.\n\n");
     }
     if ((bximage_func == BXIMAGE_FUNC_COMMIT_UNDOABLE) && (fnargs == 1)) {
       snprintf(bx_filename_2, 520, "%s%s", bx_filename_1, UNDOABLE_REDOLOG_EXTENSION);
@@ -766,6 +775,60 @@ int get_image_mode_and_hdsize(const char *filename, const char **image_mode)
   return hdsize_megs;
 }
 
+void print_partitiontable(device_image_t *hdimage)
+{
+  size_t ret = 0;
+  unsigned int n = 0;
+  Bit8u data[SECTOR_SIZE];
+
+  ret = hdimage->lseek(0, SEEK_SET);
+  if (ret < 0) {
+    fatal("lseek() to MBR failed");
+  }
+  ret = hdimage->read(data, SECTOR_SIZE);
+  if (ret != SECTOR_SIZE) {
+    fatal("failed to read MBR");
+  }
+
+  // check bootsector ID, not present = no partition table present
+  if ((data[0x1FE] != 0x55) && (data[0x1FE] != 0xAA)) {
+    printf("No partition-table detected.\n\n");
+    return;
+  }
+
+  printf("Partition table\n");
+  printf("  boot\t sector start\tsize\ttype\n");
+  for (n = 0; n < 4; n++) {
+    // calculate position of partition record (n) in table
+    Bit8u *p = &data[0x1BE + (16 * n)];
+    Bit8u bootable = p[0x00];	// 0x80 = bootable, 0x00 = not bootable
+    Bit8u type = p[0x04];
+    Bit32u start = *((Bit32u *)&p[0x08]);
+    Bit32u size  = *((Bit32u *)&p[0x0C]);
+
+    if (type > 0) {
+      // print partition number
+      printf("%d.  ", (n + 1));
+
+      // print '*' if partition is bootable
+      if (bootable) printf("*\t "); else printf("\t ");
+
+      // print start of partition in sectors
+      printf("%12u", start);
+
+      // print size of partition in sectors
+      size = (size >> 1) / 1024;
+      if (size > (4 * 1024))
+        printf("%6uG\t", (size / 1024));
+      else
+        printf("%6uM\t", size);
+
+      printf("%02X\n", type);
+    }
+  }
+  printf("\n");
+}
+
 int CDECL main(int argc, char *argv[])
 {
   char bochsrc_line[1024], prompt[80], tmpfname[528];
@@ -998,6 +1061,7 @@ int CDECL main(int argc, char *argv[])
             printf("geometry = %d/%d/%d (" FMT_LL "d MB)\n\n", hdimage->cylinders,
                    hdimage->heads, hdimage->spt, hdimage->hd_size >> 20);
           }
+          print_partitiontable(hdimage);
           hdimage->close();
         }
         break;
